/*
 * BluetoothComm.c
 *
 * main file for BluetoothComm
 *
 */
 
#include "BluetoothComm.h"
#include "BluetoothComm_Rsc.h"
#include "BtAppLib.h"
#include "UiAppLib.h"
#include "SrAppLib.h"
 
#include <PalmOS.h>
#include <PalmOSGlue.h>
#include <UI/UIPublic.h>
#include <System/SystemPublic.h>
#include <PalmCompatibility.h>
#include <PalmTypes.h>
#include <PalmOS.h>

// The actual start-point of the application,
// which also checks the OS version.
UInt32 PilotMain(UInt16 cmd, MemPtr cmdPBP, UInt16 launchFlags)
{
        UInt32 currentRomVersion;
        UInt32 requiredRomVersion;
        requiredRomVersion = 0x04000000;
        setErrorExit(0);

        FtrGet(sysFtrCreator, sysFtrNumROMVersion, &currentRomVersion);
        if (currentRomVersion < requiredRomVersion)
        {
                FrmAlert(OsVersionTooLowAlert);
                return(sysErrRomIncompatible);
        }

        if (cmd == sysAppLaunchCmdNormalLaunch)
        {
                Err returnCode;
                returnCode = startApplication();

                if (returnCode != 0)
                {
                        ErrAlert(returnCode);
                        return(returnCode);
                }
                AppEventLoop();

                returnCode = stopApplication();
                
                if (returnCode != 0 && !getErrorExit())
                {
                        ErrAlert(returnCode);
                        return(returnCode);
                }
        }

        return 0;
}

// startApplication also starts the bluetooth stack as client
Err startApplication()
{
	Err returnCode = 0;

	returnCode = startBluetoothStack();
	if ( returnCode == 0) {
		FrmGotoForm(MainForm);
	}

	return returnCode;
}

// This is the main event loop 
static void AppEventLoop(void)
{
        EventType event;
        Err error;
       
        do
        {
        		// Retrieve an event about once every 100 milisecond.
                EvtGetEvent(&event, evtWaitForever); // 10 ticks instead of "evtWaitForever"

				
                if (SysHandleEvent(&event)) {
                        continue;
                }


                if (MenuHandleEvent((void *)0, &event, &error)) {
                        continue;
                }


                if (AppHandleEvent(&event)) {
                        continue;
                }


                FrmDispatchEvent(&event);
                
        }
        while (event.eType != appStopEvent && !getErrorExit());
}

// This is the function with the business-logic
// This function handles all form events like (software) button presses
static Boolean MainFormHandleEvent(EventType *event)
{

    Boolean handled = false;
    Boolean btRequest = false;
    UInt8 waitCounter = 0;
    UInt8 maxCounter = 10;
    char* rawData = NULL;
    UInt16 tappedButton;
    
  	// The following strings must be identical to the ones in the config.php!
	char* bedroomPtr = "Bedroom"; 
	char* basementPtr =  "Basement";

    if(event->eType == frmOpenEvent) {

            FrmDrawForm(FrmGetActiveForm());
            ClearField(MainForm, DialogField);
			WriteInDialogField(MainForm, DialogField, "Please select a data point.");
            handled = true;
    }

    if(event->eType == ctlSelectEvent) {

            switch (event->data.ctlSelect.controlID)
            {

                case ClearButton:
                        ClearField(MainForm, DialogField);
                        handled = true;
                        break;
                        
                case BedroomBtn:
                		
                        SendDataSerial(bedroomPtr);
                        handled = true;
                        btRequest = true;
                        break;
                        
                case BasementBtn:
                        SendDataSerial(basementPtr);
                        handled = true;
                        btRequest = true;
                        break;
                        
                default:
                  		break;
      		}
    }
    
    if(btRequest){
    
    	ClearField(MainForm, DialogField);
    	WriteInDialogField(MainForm, DialogField, "Fetching data...");
    
	    while(!rawData){
	    
	        waitCounter++;
    		SysTaskDelay(sysTicksPerSecond/4);
	    
			// Read data from the serial port.
		    rawData = ReadSerial();
		    
			if(rawData && StrLen(rawData) > 20){
			
				ProcessSerialData(rawData);
				
			}
			
			if(getErrorExit()){
				break;
			}
			
			if(waitCounter >= maxCounter){
			    ClearField(MainForm, DialogField);
    			WriteInDialogField(MainForm, DialogField, "Fetching data... failed!");
    			
    			tappedButton = handleConnectionLost(BtErrorOccurredAlert3);
    			
    			if(tappedButton == 0 || tappedButton == 1){
    				// exit or reconnect, not more action here required
					break;
				}
    			
				if(tappedButton == 2){
					// retry
					ClearField(MainForm, DialogField);
    				WriteInDialogField(MainForm, DialogField, "Retrying to fetch data...");
					
					waitCounter = 0;
					maxCounter = 20;
				}
				

				
			}
		}
		

		
	}
    
    return handled;
}

// Handle all incoming events
static Boolean AppHandleEvent(EventPtr event)
{
        FormType* formTypePtr;
        UInt16 formId;
        Boolean handled = false;

        if (event->eType == frmLoadEvent)
        {

                formId = event->data.frmLoad.formID;
                formTypePtr = FrmInitForm(formId);
                FrmSetActiveForm(formTypePtr);

                switch (formId)
                {
                case MainForm:
                        FrmSetEventHandler(formTypePtr, MainFormHandleEvent);
                        break;


                default:
                        break;
                }

                return true;
        }

        if (event->eType == menuEvent)
        {
                MenuEraseStatus(NULL);
                switch (event->data.menu.itemID)
                {
                case MainOptionsHelpCmd:
                        FrmAlert(HelpAlert);
                        handled = true;
                        break;

                case MainOptionsAboutCmd:
                        FrmAlert(AboutApplicationAlert);
                        handled = true;
                        break;

                default:
                        break;
                }

                return true;
        }

        return false;
}

// stop the application and close all open ports
Err stopApplication()
{
        Err error;
        UInt16 btPort = getBtPort();
         
        ClearField(MainForm, DialogField);
		WriteInDialogField(MainForm, DialogField, "Shutting down bluetooth stack...");
       
		error = SrmSendWait(btPort);
		ErrNonFatalDisplayIf(error == serErrBadPort,"SrmClose: bad port");
		if (error == serErrTimeOut){
			FrmAlert(ErrorOccurredAlert);
		}
		SrmClose(btPort);
        FrmCloseAllForms();
        return error;
}

// Triggers an exit if an error occurs (if ErrorExit is not zero)
Err getErrorExit(){

	UInt32 error = 0;

	FtrGet(appFileCreator, 10110, &error);
	
	return error;
}
void setErrorExit(UInt32 error){

	FtrSet(appFileCreator, 10110, error);
}

// This function proccesses the serial data, which gets received by the ESP32
static void ProcessSerialData(char* rawData){

	UInt8 i;
	UInt8 c = 0;
	UInt8 j = 0;
	UInt8 k = 0;

	char complete[300] = "";
	char time[30] = "";
	char temp[15] = "";
	char hum[15] = "";
	
	// Not beautiful, but it is working.
	// I'm not sure if there is something like "explode(';',rawData);"
	// Since, every data has a fixed length, 
	// we can work with fixed values, too. 
	for(i = 0; i < StrLen(rawData); i++){

		if( ';' == rawData[i]){
			c++;
		}

		if(c == 0 && rawData[i] != ';' && i < 20){
			time[i] = rawData[i];
		}

		if(c == 1 && rawData[i] != ';' && j < 5){
			hum[j] += rawData[i];
			j++;
		}

		if(c == 2 && rawData[i] != ';' && k < 5){
			temp[k] = rawData[i];
			k++;
		}
	}
	  
	StrCat(complete, "Time: ");
	StrCat(complete, time);
	StrCat(complete, "\n");
	StrCat(complete, "Temperature: ");
	StrCat(complete, temp);
	StrCat(complete, "C\n");
	StrCat(complete, "Humidity: ");
	StrCat(complete, hum);
	StrCat(complete, "%\n");
	
	ClearField(MainForm, DialogField);
	WriteInDialogField(MainForm, DialogField, complete);	
}
